/*
* DemoSoftware.c
*
* Created: 23.06.2017
* Author : Dominik Iten
*/

//#define F_CPU 16000000UL

#include <avr/io.h>
#include "hd44780.h"
#include "Formel_NTC10k_ref10k_RN001.h"
#include <util/delay.h>
#include <avr/interrupt.h>

#define UART_BAUD_RATE      9600   

#include <util/twi.h> //=> I2C

//PORT-Names

#define LED			PORTA
#define SCHALTER	PINC
#define _7SEG		PORTH
#define TASTER		PINJ
#define LCD			PORTL

//RGB Color Definitions

#define RGB_MaxPWMValue 255
#define RGB_Intensity   5     //in %

//Red
#define RED_R             RGB_MaxPWMValue * RGB_Intensity / 100
#define RED_G             0
#define RED_B
const char Red[3] = {RED_R, RED_G, RED_B};

//Yellow
#define Yellow_R          RGB_MaxPWMValue * RGB_Intensity / 200
#define Yellow_G          RGB_MaxPWMValue * RGB_Intensity / 200
#define Yellow_B          0
const char Yellow[3] = {Yellow_R, Yellow_G, Yellow_B};

//Green
#define Green_R           0
#define Green_G           RGB_MaxPWMValue * RGB_Intensity / 100
#define Green_B           0
const char Green[3] = {Green_R, Green_G, Green_B};

//Cyan
#define Cyan_R           0
#define Cyan_G           RGB_MaxPWMValue * RGB_Intensity / 200
#define Cyan_B           RGB_MaxPWMValue * RGB_Intensity / 200
const char Cyan[3] = {Cyan_R, Cyan_G, Cyan_B};

//Blue
#define Blue_R           0
#define Blue_G           0
#define Blue_B           RGB_MaxPWMValue * RGB_Intensity / 100
const char Blue[3] = {Blue_R, Blue_G, Blue_B};

//Violet
#define Violet_R           RGB_MaxPWMValue * RGB_Intensity / 200
#define Violet_G           0
#define Violet_B           RGB_MaxPWMValue * RGB_Intensity / 200
const char Violet[3] = {Violet_R, Violet_G, Violet_B};


//Funktion-Makros

#define BUZZER_ON	(PORTD |= 0x80)
#define BUZZER_OFF	(PORTD &= 0x7F)

#define _7SEG_EN		(PORTG |= 0x04)
#define _7SEG_DIS		(PORTG &= ~0x04)

#define _7SEG_COLON_ON	(PORTG |= 0x10)
#define _7SEG_COLON_OFF	(PORTG &= ~0x10)

#define _7SEG_COMMA_ON	(PORTG |= 0x08)
#define _7SEG_COMMA_OFF	(PORTG &= ~0x08)

#define HEAT_ON			(PORTB |= 0x10)
#define HEAT_OFF		(PORTB &= ~0x10)

#define _7SEG_CURRENT_POS(Pos)   (PORTG = (PORTG & 0xFC) + Pos)


//Function-Prototypes
void ADC_start_conversion(char ADC_Number);
void ADC_get_Buffervalue(char ADC_Number);
void ADC_get_all_Buffervalues(void);
void SysClk(void);
void _7_SEG_Int_to_Display(unsigned int Value);


//Constant-Data
const char _7SEG_conversion[10] = {0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F};	//Converts a Character in to a 8Bit Value for Output to 7Seg-Display
//to add DP:	Result |= 0x80


//Global Flags & Variables
char ADC_conversion_in_progress;
unsigned int ADC_All_Buffers[16] = {0};
char _7SEG_Buffer[4] = {0x06, 0x5B, 0x4F, 0x3F};
char _7SEG_Pos = 0;

char LD_Charly = 0;

//SysClk-Flags
char SysClk_1ms;
char SysClk_2ms;
char SysClk_5ms;
char SysClk_10ms;
char SysClk_20ms;
char SysClk_100ms;
char SysClk_500ms;
char SysClk_1000ms;


//Definition of Content ADC-Channels

//ADC0 is connected to OC 0 (usable but impractical)
#define U_MEASURE			1
#define NTC					2
#define POTI				3
//ADC4-7 are connected on the JTAG programming Header (usable but impractical)
//ADC8-15 can be used for external Measurements





int main(void)
{
    //Define all Data Direction Registers
    //	All DDR must be set accordingly as else there could be inexplainable Events occurring on the Microcontroller-Board
    DDRA = 0xFF;	// LEDs as Output
    DDRB = 0xF7;
    DDRC = 0x00;	// Schalter as Input
    DDRD = 0x88;
    DDRE = 0x1A;
    DDRF = 0x00;
    DDRG = 0xFF;
    DDRH = 0xFF;	// 7Segment as Output
    DDRJ = 0x00;	// Taster as Input
    DDRK = 0xFF;
    DDRL = 0xFF;	// LCD as Output
    
    /*
    //ADC-Config
    ADMUX = 0b00000000;
    ADCSRA = 0xEA;
    ADCSRB = 0x00;
    */
    
    lcd_init();
    //PORTL |= 0x08;	        //Backlight on
    _7SEG_EN;
    //HEAT_ON;
    BUZZER_ON;
    _delay_ms(10);
    BUZZER_OFF;
    

    //Start System-Clock Timer (TCNT 5)
    TCCR5A = 0b00000000;	//CS div64 -> 1Clk = 4us, no PWM, Clear on Compare Match
    TCCR5B = 0b00001011;
    TIMSK5 = 0b00000010;	//Output Compare Interrupt enable
    OCR5A  = 250;			//-> Clear after 1ms @16MHz = 250, @8MHz = 125, @1MHz ~ 16
    
    //RGB Init (TCNT1)
    TCCR1A = 0b10101001;    //no prescaling, all Interrupts disabled
    TCCR1B = 0b00001001;
    TIMSK1 = 0b00000000;
    
    
    sei();					//enable all Interrupts

    while(1)
    {
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_1ms)
        {
            SysClk_1ms = 0;
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_2ms)
        {
            SysClk_2ms = 0;
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_5ms)
        {
            SysClk_5ms = 0;
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_10ms)
        {
            SysClk_10ms = 0;
            
            LED = ~SCHALTER;
            if(TASTER)
            {
                //BUZZER_ON;
            }
            else
            {
                //BUZZER_OFF;
            }
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_20ms)
        {
            SysClk_20ms = 0;
            
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_100ms)
        {
            static unsigned int Counter = 0;
            
            SysClk_100ms = 0;
            
            //ADC_start_conversion(POTI);
            //ADC_get_Buffervalue(POTI);
            //_7_SEG_Int_to_Display(ADC_All_Buffers[POTI]);
            
            _7_SEG_Int_to_Display(Counter);
            Counter++;
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_500ms)
        {
            SysClk_500ms = 0;
            
            LD_Charly++;
            
            switch (LD_Charly)
            {
                case 1:
                DDRK &= 0xFE;   //LD6
                DDRK |= 0x06;
                PORTK = 0x02;
                break;
                
                case 2:
                DDRK &= 0xFE;   //LD5
                DDRK |= 0x06;
                PORTK = 0x04;
                break;
                
                case 3:
                DDRK &= 0xFD;   //LD4
                DDRK |= 0x05;
                PORTK = 0x01;
                break;
                
                case 4:
                DDRK &= 0xFD;   //LD3
                DDRK |= 0x05;
                PORTK = 0x04;
                break;
                
                case 5:
                DDRK &= 0xFB;   //LD2
                DDRK |= 0x03;
                PORTK = 0x02;
                break;
                
                case 6:
                DDRK &= 0xFB;   //LD1
                DDRK |= 0x03;
                PORTK = 0x01;
                LD_Charly = 0;
                break;
            }
            
            
            
            
        }
        
        //////////////////////////////////////////////////////////////////////////////////////
        if(SysClk_1000ms)
        {
            static char Count_ColorWeel = 0;
            static char LineCount = 0;
            
            SysClk_1000ms = 0;
            
            //lcd_clrscr();
            if(LineCount == 0)
            {
                lcd_goto(0x00);
                lcd_puts("DEMO:");
                LineCount = 1;
            }
            else
            {
                lcd_goto(0x40);
                lcd_puts("Testsoftware");
                LineCount = 0;
            }
            
            //lcd_goto(0x00);
            //lcd_puts("0123456789ABCDEF");
            //lcd_goto(0x40);
            //lcd_puts("0123456789ABCDEF");
            //PORTK = 0x00;
            switch(Count_ColorWeel)
            {
                case 0:
                OCR1A = Red[0];
                OCR1B = Red[1];
                OCR1C = Red[2];
                break;
                
                case 1:
                OCR1A = Yellow[0];
                OCR1B = Yellow[1];
                OCR1C = Yellow[2];
                break;
                
                case 2:
                OCR1A = Green[0];
                OCR1B = Green[1];
                OCR1C = Green[2];
                break;
                
                case 3:
                OCR1A = Cyan[0];
                OCR1B = Cyan[1];
                OCR1C = Cyan[2];
                break;
                
                case 4:
                OCR1A = Blue[0];
                OCR1B = Blue[1];
                OCR1C = Blue[2];
                break;
                
                case 5:
                OCR1A = Violet[0];
                OCR1B = Violet[1];
                OCR1C = Violet[2];
                Count_ColorWeel = -1;
                break;
            }
            Count_ColorWeel++;
            
            
        }
    }
}



ISR(TIMER5_COMPA_vect)
{
    static char DigitCount = 0;
    
    SysClk();
    _7SEG_DIS;
    DigitCount = (DigitCount + 1) & 0x03;
    _7SEG_CURRENT_POS(DigitCount);
    _7SEG = _7SEG_Buffer[DigitCount];
    _7SEG_EN;
}


void SysClk(void)
{

    // Static Variables for Counter
    static char Counter_2ms = 0;
    static char Counter_5ms = 0;
    static char Counter_10ms = 0;
    static char Counter_20ms = 0;
    static char Counter_100ms = 0;
    static char Counter_500ms = 0;
    static char Counter_1000ms = 0;

    //////////////////////////////////////////////////////////////////////////////
    // Counter 1ms
    //////////////////////////////////////////////////////////////////////////////
    
    SysClk_1ms = 1;

    //////////////////////////////////////////////////////////////////////////////
    // Counter 3ms
    //////////////////////////////////////////////////////////////////////////////
    
    Counter_2ms++;
    if( Counter_2ms >= 2 ){
        Counter_2ms = 0;
        SysClk_2ms = 1;
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 5ms
    //////////////////////////////////////////////////////////////////////////////
    
    Counter_5ms++;
    if( Counter_5ms >= 5 ){
        Counter_5ms = 0;
        SysClk_5ms = 1;
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 10ms
    //////////////////////////////////////////////////////////////////////////////
    
    Counter_10ms++;
    if( Counter_10ms >= 10 ){
        Counter_10ms = 0;
        SysClk_10ms = 1;
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 20ms
    //////////////////////////////////////////////////////////////////////////////
    
    Counter_20ms++;
    if( Counter_20ms >= 20 ){
        Counter_20ms = 0;
        SysClk_20ms = 1;
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 100ms
    //////////////////////////////////////////////////////////////////////////////
    
    Counter_100ms++;
    if( Counter_100ms >= 100 ){
        Counter_100ms = 0;
        SysClk_100ms = 1;
        Counter_500ms++;
        Counter_1000ms++;
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 500ms
    //////////////////////////////////////////////////////////////////////////////
    
    if( Counter_500ms >= 5 ){
        Counter_500ms = 0;
        SysClk_500ms = 1;
        
    }
    //////////////////////////////////////////////////////////////////////////////
    // Counter 1000ms
    //////////////////////////////////////////////////////////////////////////////
    
    if( Counter_1000ms >= 10 ){
        Counter_1000ms = 0;
        SysClk_1000ms = 1;
    }

}


void ADC_start_conversion(char ADC_Number)
{
    if(ADC_Number < 8)
    {
        ADCSRB &= ~0x08;		//clear MUX5
        ADMUX  &= ~0x1F;		//clear MUX0-4
        ADMUX  += ADC_Number;	//set MUX0-4 according to ADC_Number
    }
    else
    {
        ADCSRB |= 0x08;			//set MUX5
        ADMUX  &= ~0x1F;		//clear MUX0-4
        ADMUX  += ADC_Number-8;	//set MUX0-4 according to ADC_Number
    }
    ADC_conversion_in_progress = 1;
    while(ADC_conversion_in_progress)
    {}
}


void ADC_get_Buffervalue(char ADC_Number)
{
    //ADC_All_Buffers[ADC_Number] = ADCL + ((ADCH&~0x03) <<8);
    ADC_All_Buffers[ADC_Number] = ADC;
}

void ADC_get_all_Buffervalues(void)
{
    char ADC_Number;
    for(ADC_Number = 0; ADC_Number < 16; ADC_Number++)
    {
        ADC_start_conversion(ADC_Number);
        ADC_get_Buffervalue(ADC_Number);
    }
}

ISR(ADC_vect)
{
    ADC_conversion_in_progress = 0;
}

void _7_SEG_Int_to_Display(unsigned int Value)
{
    //Last 4 Digits of Value saved and Displayed on 7-Segment Display
    
    //for()

    for(char i = 0; i < 4; i++)
    {
        _7SEG_Buffer[i] = _7SEG_conversion[Value%10];
        Value /= 10;
    }
    
    //_7SEG_Buffer[0] = _7SEG_conversion[Value%10];
    //_7SEG_Buffer[1] = _7SEG_conversion[Value%100/10];
    //_7SEG_Buffer[2] = _7SEG_conversion[Value%1000/100];
    //_7SEG_Buffer[3] = _7SEG_conversion[Value%10000/1000];
}


